/*
    Copyright (C) 2006 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <cssysdef.h>
#include <crystalspace.h>
#include "appmazing.h"
#include "gamestuff.h"

//-----------------------------------------------------------------------------

Maze::Maze (AppMazing* app)
{
  Maze::app = app;
  int x, y, z;
  for (x = 0 ; x < MAZE_DIMENSION ; x++)
    //crhz for (y = 0 ; y < MAZE_DIMENSION ; y++)
    y=0;
      for (z = 0 ; z < MAZE_DIMENSION ; z++)
        occupied[x][y][z] = false;
}

bool Maze::CreateSector (int x, int y, int z)
{
  CS_ASSERT (RoomCoordinate::IsValid (x, y, z));
  char sector_name[100];
  sprintf (sector_name, "room_%d_%d_%d", x, y, z);
  rooms[x][y][z] = app->GetEngine ()->CreateSector (sector_name);

  return rooms[x][y][z] != 0;
}

bool Maze::CreateSectors ()
{
  int x, z;
  for (x = 0 ; x < MAZE_DIMENSION ; x++)
    //crhz make it 2d for (y = 0 ; y < MAZE_DIMENSION ; y++)
    for (z = 0 ; z < MAZE_DIMENSION ; z++)
      if (!CreateSector (x, 0, z))
      return false;
  return true;
}

void Maze::MakeConnection (const RoomCoordinate& from, const RoomCoordinate& to)
{
  CS_ASSERT (from.IsValid ());
  CS_ASSERT (to.IsValid ());
  connections[from.x][from.y][from.z].Push (to);
}

void Maze::FreeSpace (const RoomCoordinate& rc)
{
  CS_ASSERT (rc.IsValid ());
  occupied[rc.x][rc.y][rc.z] = false;
}

void Maze::OccupySpace (const RoomCoordinate& rc)
{
  CS_ASSERT (rc.IsValid ());
  occupied[rc.x][rc.y][rc.z] = true;
}

bool Maze::IsSpaceFree (const RoomCoordinate& rc) const
{
  CS_ASSERT (rc.IsValid ());
  return !occupied[rc.x][rc.y][rc.z];
}

bool Maze::CreateWallOrPortal (iThingFactoryState* factory_state,
  	const csVector3& v1, const csVector3& v2,
  	const csVector3& v3, const csVector3& v4,
	bool do_portal,
	const RoomCoordinate& source,
	const RoomCoordinate& dest)
{
  if (do_portal)
  {
    iPortal* portal;
    csVector3 verts[4];
    verts[0] = v1;
    verts[1] = v2;
    verts[2] = v3;
    verts[3] = v4;
    csRef<iMeshWrapper> portal_mesh = app->GetEngine ()->CreatePortal (0,
    	GetSector (source), csVector3 (0, 0, 0),
	GetSector (dest),
    	verts, 4, portal);
    MakeConnection (source, dest);
    if (!portal_mesh)
      return app->ReportError ("Error creating portal mesh!");
  }
  else
  {
    factory_state->AddQuad (v1, v2, v3, v4);
  }
  return true;
}

bool Maze::CreateRoom (iMaterialWrapper* wall_material,
	int x, int y, int z, char* portals)
{
  iSector* room = GetSector (x, y, z);
  csRef<iMeshWrapper> walls = app->GetEngine ()
  	->CreateSectorWallsMesh (room, "walls");
  if (!walls)
    return app->ReportError ("Couldn't create the walls for the room!");

  csRef<iThingState> object_state = scfQueryInterface<iThingState> (
  	walls->GetMeshObject ());
  csRef<iThingFactoryState> factory_state = scfQueryInterface<
  	iThingFactoryState> (walls->GetMeshObject ()->GetFactory ());
  float sx = float (x) * ROOM_DIMENSION;
  float sy = float (y) * ROOM_DIMENSION;
  float sz = float (z) * ROOM_DIMENSION;
  float rd = ROOM_DIMENSION / 2.0;
  csBox3 box (
  	csVector3 (sx-rd, sy-rd, sz-rd),
  	csVector3 (sx+rd, sy+rd, sz+rd));

  RoomCoordinate start_rc (x, y, z);
  if (!CreateWallOrPortal (factory_state, 
    	box.GetCorner (CS_BOX_CORNER_xYz),
    	box.GetCorner (CS_BOX_CORNER_XYz),
    	box.GetCorner (CS_BOX_CORNER_XYZ),
    	box.GetCorner (CS_BOX_CORNER_xYZ),
        false,
        //portals[PORTAL_UP] == '#',
	start_rc, RoomCoordinate (x+0, y+1, z+0)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_xyZ),
    	box.GetCorner (CS_BOX_CORNER_XyZ),
    	box.GetCorner (CS_BOX_CORNER_Xyz),
    	box.GetCorner (CS_BOX_CORNER_xyz),
        false,
        //portals[PORTAL_DOWN] == '#',
	start_rc, RoomCoordinate (x+0, y-1, z+0)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_XyZ),
    	box.GetCorner (CS_BOX_CORNER_xyZ),
    	box.GetCorner (CS_BOX_CORNER_xYZ),
    	box.GetCorner (CS_BOX_CORNER_XYZ),
	portals[PORTAL_FRONT] == '#',
	start_rc, RoomCoordinate (x+0, y+0, z+1)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_xyz),
    	box.GetCorner (CS_BOX_CORNER_Xyz),
    	box.GetCorner (CS_BOX_CORNER_XYz),
    	box.GetCorner (CS_BOX_CORNER_xYz),
	portals[PORTAL_BACK] == '#',
	start_rc, RoomCoordinate (x+0, y+0, z-1)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_xyZ),
    	box.GetCorner (CS_BOX_CORNER_xyz),
    	box.GetCorner (CS_BOX_CORNER_xYz),
    	box.GetCorner (CS_BOX_CORNER_xYZ),
	portals[PORTAL_LEFT] == '#',
	start_rc, RoomCoordinate (x-1, y+0, z+0)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_Xyz),
    	box.GetCorner (CS_BOX_CORNER_XyZ),
    	box.GetCorner (CS_BOX_CORNER_XYZ),
    	box.GetCorner (CS_BOX_CORNER_XYz),
	portals[PORTAL_RIGHT] == '#',
	start_rc, RoomCoordinate (x+1, y+0, z+0)))
    return false;

  factory_state->SetPolygonMaterial (CS_POLYRANGE_ALL, wall_material);
  factory_state->SetPolygonTextureMapping (CS_POLYRANGE_ALL, rd);

  return true;
}

bool Maze::MyCreateRoom (int x, int y, int z, char* portals)
{
  // Load the texture we are going to use for all walls.
  if (!app->GetLoader ()->LoadTexture ("w2all_texture", "/lib/std/stone4.gif"))
    return app->ReportError ("Error loading 'stone4' texture!");

  iMaterialWrapper* wall_material = app->GetEngine ()->GetMaterialList ()
  	->FindByName ("w2all_texture");

  iSector* room = GetSector (x, y, z);
  csRef<iMeshWrapper> walls = app->GetEngine ()
  	->CreateSectorWallsMesh (room, "walls");
  if (!walls)
    return app->ReportError ("Couldn't create the walls for the room!");

  csRef<iThingState> object_state = scfQueryInterface<iThingState> (
  	walls->GetMeshObject ());
  csRef<iThingFactoryState> factory_state = scfQueryInterface<
  	iThingFactoryState> (walls->GetMeshObject ()->GetFactory ());
  float sx = float (x) * ROOM_DIMENSION;
  float sy = float (y) * ROOM_DIMENSION;
  float sz = float (z) * ROOM_DIMENSION;
  float rd = ROOM_DIMENSION / 2.0;
  csBox3 box (
  	csVector3 (sx-rd, sy-rd, sz-rd),
  	csVector3 (sx+rd, sy+rd, sz+rd));

  RoomCoordinate start_rc (x, y, z);
  if (!CreateWallOrPortal (factory_state, 
    	box.GetCorner (CS_BOX_CORNER_xYz),
    	box.GetCorner (CS_BOX_CORNER_XYz),
    	box.GetCorner (CS_BOX_CORNER_XYZ),
    	box.GetCorner (CS_BOX_CORNER_xYZ),
        false,
        //portals[PORTAL_UP] == '#',
	start_rc, RoomCoordinate (x+0, y+1, z+0)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_xyZ),
    	box.GetCorner (CS_BOX_CORNER_XyZ),
    	box.GetCorner (CS_BOX_CORNER_Xyz),
    	box.GetCorner (CS_BOX_CORNER_xyz),
        false,
        //portals[PORTAL_DOWN] == '#',
	start_rc, RoomCoordinate (x+0, y-1, z+0)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_XyZ),
    	box.GetCorner (CS_BOX_CORNER_xyZ),
    	box.GetCorner (CS_BOX_CORNER_xYZ),
    	box.GetCorner (CS_BOX_CORNER_XYZ),
	portals[PORTAL_FRONT] == '#',
	start_rc, RoomCoordinate (x+0, y+0, z+1)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_xyz),
    	box.GetCorner (CS_BOX_CORNER_Xyz),
    	box.GetCorner (CS_BOX_CORNER_XYz),
    	box.GetCorner (CS_BOX_CORNER_xYz),
	portals[PORTAL_BACK] == '#',
	start_rc, RoomCoordinate (x+0, y+0, z-1)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_xyZ),
    	box.GetCorner (CS_BOX_CORNER_xyz),
    	box.GetCorner (CS_BOX_CORNER_xYz),
    	box.GetCorner (CS_BOX_CORNER_xYZ),
	portals[PORTAL_LEFT] == '#',
	start_rc, RoomCoordinate (x-1, y+0, z+0)))
    return false;
  if (!CreateWallOrPortal (factory_state,
    	box.GetCorner (CS_BOX_CORNER_Xyz),
    	box.GetCorner (CS_BOX_CORNER_XyZ),
    	box.GetCorner (CS_BOX_CORNER_XYZ),
    	box.GetCorner (CS_BOX_CORNER_XYz),
	portals[PORTAL_RIGHT] == '#',
	start_rc, RoomCoordinate (x+1, y+0, z+0)))
    return false;

  factory_state->SetPolygonMaterial (CS_POLYRANGE_ALL, wall_material);
  factory_state->SetPolygonTextureMapping (CS_POLYRANGE_ALL, rd);

  return true;
}

bool Maze::CreateRooms () {

  // Load the texture we are going to use for all walls.
  if (!app->GetLoader ()->LoadTexture ("wall_texture", "/lib/std/tile.png"))
    return app->ReportError ("Error loading 'tile' texture!");
  iMaterialWrapper* wall_material = app->GetEngine ()->GetMaterialList ()
  	->FindByName ("wall_texture");

  // Load the texture we are going to use for corridors.
  if (!app->GetLoader()->LoadTexture ("wall2_texture", "/lib/std/asphalt.jpg"))
    return app->ReportError ("Error loading 'stone4' texture!");

  iMaterialWrapper* wall2_material = app->GetEngine ()->GetMaterialList ()
  	->FindByName ("wall2_texture");

  //crhzif (!MyCreateRoom (0, 0, 0, "...##.")) return false;
  if (!CreateRoom (wall_material, 0, 0, 0, "...##.")) return false;
  if (!CreateRoom (wall_material, 0, 0, 1, "...###")) return false;
  if (!CreateRoom (wall_material, 0, 0, 2, "...#.#")) return false;
  if (!CreateRoom (wall_material, 0, 0, 3, "......")) return false;
  if (!CreateRoom (wall_material, 0, 0, 4, "...##.")) return false;
  if (!CreateRoom (wall_material, 0, 0, 5, "...###")) return false;
  if (!CreateRoom (wall_material, 0, 0, 6, "...#.#")) return false;
  if (!CreateRoom (wall_material, 1, 0, 0, "..###.")) return false;
  if (!CreateRoom (wall_material, 1, 0, 1, "..####")) return false;
  if (!CreateRoom (wall_material, 1, 0, 2, "..####")) return false;
  if (!CreateRoom (wall2_material, 1, 0, 3, "....##")) return false;
  if (!CreateRoom (wall_material, 1, 0, 4, "..####")) return false;
  if (!CreateRoom (wall_material, 1, 0, 5, "..####")) return false;
  if (!CreateRoom (wall_material, 1, 0, 6, "..##.#")) return false;
  if (!CreateRoom (wall_material, 2, 0, 0, "..#.#.")) return false;
  if (!CreateRoom (wall_material, 2, 0, 1, "..####")) return false;
  if (!CreateRoom (wall_material, 2, 0, 2, "..#..#")) return false;
  if (!CreateRoom (wall_material, 2, 0, 3, "......")) return false;
  if (!CreateRoom (wall_material, 2, 0, 4, "..#.#.")) return false;
  if (!CreateRoom (wall_material, 2, 0, 5, "..####")) return false;
  if (!CreateRoom (wall_material, 2, 0, 6, "..#..#")) return false;
  if (!CreateRoom (wall_material, 3, 0, 0, "......")) return false;
  if (!CreateRoom (wall2_material, 3, 0, 1, "..##..")) return false;
  if (!CreateRoom (wall_material, 3, 0, 2, ".....")) return false;
  if (!CreateRoom (wall_material, 3, 0, 3, ".....")) return false;
  if (!CreateRoom (wall_material, 3, 0, 4, ".....")) return false;
  if (!CreateRoom (wall2_material, 3, 0, 5, "..##..")) return false;
  if (!CreateRoom (wall_material, 3, 0, 6, "......")) return false;
  if (!CreateRoom (wall_material, 4, 0, 0, "...##.")) return false;
  if (!CreateRoom (wall_material, 4, 0, 1, "..####")) return false;
  if (!CreateRoom (wall_material, 4, 0, 2, "...#.#")) return false;
  if (!CreateRoom (wall_material, 4, 0, 3, "......")) return false;
  if (!CreateRoom (wall_material, 4, 0, 4, "...##.")) return false;
  if (!CreateRoom (wall_material, 4, 0, 5, "..####")) return false;
  if (!CreateRoom (wall_material, 4, 0, 6, "...#.#")) return false;
  if (!CreateRoom (wall_material, 5, 0, 0, "..###.")) return false;
  if (!CreateRoom (wall_material, 5, 0, 1, "..####")) return false;
  if (!CreateRoom (wall_material, 5, 0, 2, "..####")) return false;
  if (!CreateRoom (wall2_material, 5, 0, 3, "....##")) return false;
  if (!CreateRoom (wall_material, 5, 0, 4, "..####")) return false;
  if (!CreateRoom (wall_material, 5, 0, 5, "..####")) return false;
  if (!CreateRoom (wall_material, 5, 0, 6, "..##.#")) return false;
  if (!CreateRoom (wall_material, 6, 0, 0, "..#.#.")) return false;
  if (!CreateRoom (wall_material, 6, 0, 1, "..#.##")) return false;
  if (!CreateRoom (wall_material, 6, 0, 2, "..#..#")) return false;
  if (!CreateRoom (wall_material, 6, 0, 3, "......")) return false;
  if (!CreateRoom (wall_material, 6, 0, 4, "..#.#.")) return false;
  if (!CreateRoom (wall_material, 6, 0, 5, "..#.##")) return false;
  if (!CreateRoom (wall_material, 6, 0, 6, "..#..#")) return false;

  return true;
}

bool Maze::CreateLight (const csColor& color, int x, int y, int z)
{
  float sx = float (x) * ROOM_DIMENSION;
  float sy = float (y) * ROOM_DIMENSION;
  float sz = float (z) * ROOM_DIMENSION;
  csRef<iLight> light = app->GetEngine ()
  	->CreateLight (0, csVector3 (sx, sy, sz), ROOM_DIMENSION * 1.5, color);
  if (!light) return false;
  GetSector (x, y, z)->GetLights ()->Add (light);

  sx -= 4;
  sz -= 4;
  light = app->GetEngine ()
  	->CreateLight (0, csVector3 (sx, sy, sz), ROOM_DIMENSION * 1.5, color);
  if (!light) return false;
  GetSector (x, y, z)->GetLights ()->Add (light);

  sz += 8;
  light = app->GetEngine ()
  	->CreateLight (0, csVector3 (sx, sy, sz), ROOM_DIMENSION * 1.5, color);
  if (!light) return false;
  GetSector (x, y, z)->GetLights ()->Add (light);

  sx += 8;
  light = app->GetEngine ()
  	->CreateLight (0, csVector3 (sx, sy, sz), ROOM_DIMENSION * 1.5, color);
  if (!light) return false;
  GetSector (x, y, z)->GetLights ()->Add (light);

  sz -= 8;
  light = app->GetEngine ()
  	->CreateLight (0, csVector3 (sx, sy, sz), ROOM_DIMENSION * 1.5, color);
  if (!light) return false;
  GetSector (x, y, z)->GetLights ()->Add (light);
  return true;
}

bool Maze::CreateLights () {
  float ltStrength = 0.5;
  float dkLtStrength = 0.005;
  for (int x= 0; x<MAZE_DIMENSION;x++) {
    for (int z= 0; z<MAZE_DIMENSION;z++) {
      if (((x == 3) && (z == 1)) ||
          ((x == 3) && (z == 5)) ||
          ((x == 1) && (z == 3)) ||
          ((x == 5) && (z == 3))) //coridors
        {
        if (!CreateLight(csColor(dkLtStrength,dkLtStrength,dkLtStrength),x,0,
          z))
          return false;
	}
      else if (!CreateLight (csColor (ltStrength,ltStrength,ltStrength),
        x,0,z)) //rooms
        return false;
    }
  }
  return true;
}


bool Maze::CreateGeometry ()
{
  // We don't need a lighting cache. Disable it.
  app->GetEngine ()->SetLightingCacheMode (0);

  if (!CreateSectors()) return false;
  if (!CreateRooms()) return false;
  if (!CreateLights()) return false;

  return true;
}

//-----------------------------------------------------------------------------

Player::Player (AppMazing* app)
{
  Player::app = app;

  desired_location.Set (0, 0, 0);
  desired_lookat.Set (0, 0, 1);
  start_location.Set (10, 0, 10);
  //start_location.Set (0, 0, 0);
  start_lookat.Set (0, 0, 1);
  amount_moved = 0;
  amount_rotated = 0;
}

bool Player::InitCollisionDetection ()
{
  float ps = PLAYER_SIZE / 2.0;
  csTriangleMeshBox* box = new csTriangleMeshBox (csBox3 (
  	csVector3 (-ps, -ps, -ps), csVector3 (ps, ps, ps)));
  player_collider = app->GetCollisionDetectionSystem ()
  	->CreateCollider (box);
  box->DecRef ();

  if (player_collider == 0) return false;
  return true;
}

bool Game::InitAgentCollisionDetection ()
{
  //float ps = PLAYER_SIZE / 2.0;
  float ps =  2.0;
  csTriangleMeshBox* box = new csTriangleMeshBox (csBox3 (
  //    csVector3 (-ps, -ps, -ps), csVector3 (ps, ps, ps)));
    csVector3 (-ps, -ps, 0), csVector3 (ps, ps, ps)));
  agent_collider = app->GetCollisionDetectionSystem ()
  	->CreateCollider (box);
  box->DecRef ();

  if (agent_collider == 0) return false;
  return true;
}

void Player::MoveAndRotateCamera (float elapsed_seconds)
{
  iCamera* camera = app->GetCamera ();

  // First we move the camera.
  amount_moved += MOVECAMERA_SPEED * elapsed_seconds;
  //crhz  float move_val = 1 - pow (2.0f, -amount_moved);
  float move_val = 0.3;
  csVector3 current_pos = camera->GetTransform ().GetOrigin ();
  csVector3 new_pos = start_location * (1-move_val)
  	+ desired_location * move_val;
  camera->MoveWorld (new_pos - current_pos, false);

  // Now we rotate the camera.
  amount_rotated += ROTATECAMERA_SPEED * elapsed_seconds;
  //crhz float rotate_val = 1 - pow (2.0f, -amount_rotated);
  float rotate_val = 0.7;
  csVector3 new_lookat = start_lookat * (1-rotate_val)
  	+ desired_lookat * rotate_val;
  camera->GetTransform ().LookAt (new_lookat, csVector3 (0, 1, 0));
}

void Player::StartMovement (const csVector3& dir,iMeshWrapper* user )
{
  iCamera* camera = app->GetCamera ();

  amount_moved = 0;
  csReversibleTransform camera_trans = camera->GetTransform ();
  start_location = camera_trans.GetOrigin ();

  // Calculate the spot where we want to move too depending on elapsed
  // time and current direction the camera is facing.
  csVector3 world_dir = camera_trans.This2OtherRelative (dir);
  desired_location = start_location + MOVE_DISTANCE * world_dir;

  // First we find all meshes that are near the path that we want to move.
  // To do that we take the center point of our movement vector and
  // calculate all objects that touch the circle around that point with
  // radius (MOVE_DISTANCE+PLAYER_SIZE) / 2.0.
  csRef<iMeshWrapperIterator> it = app->GetEngine ()->GetNearbyMeshes (
  	camera->GetSector (), (start_location + desired_location) / 2.0,
	(MOVE_DISTANCE + PLAYER_SIZE) / 2.0);

  if (!it->HasNext ())
  {
    // We have no objects in the iterator so there can be no collision.
    // We can move freely.
    return;
  }

  // If we have meshes then we will calculate collision detection for our
  // object along the path we want to move.
  desired_location = start_location;
  csVector3 test_location = start_location + STEP_DISTANCE * world_dir;
  iCollideSystem* cdsys = app->GetCollisionDetectionSystem ();
  float current_move = STEP_DISTANCE;
  while (current_move <= MOVE_DISTANCE)
  {
    camera_trans.SetOrigin (test_location);

    // Test collision with our iterator.
    it->Reset ();
    while (it->HasNext ())
    {
      iMeshWrapper* mesh = it->Next ();
      csColliderWrapper* collide_wrap = csColliderWrapper::GetColliderWrapper (
    	  mesh->QueryObject ());
      if (collide_wrap)
      {
        csReversibleTransform mesh_trans = mesh->GetMovable ()
      	  ->GetFullTransform ();
	cdsys->ResetCollisionPairs ();
        if (cdsys->Collide (
      	      collide_wrap->GetCollider (), &mesh_trans,
	      player_collider, &camera_trans))
        {
	  // Collision, so we can stop. 'desired_location' will contain
	  // the last valid location that we can move too.

	  return;
	}
      }
    }
    desired_location = test_location;
    test_location += STEP_DISTANCE * world_dir;
    current_move += STEP_DISTANCE;
  }

  //move the sprite
  csRef<iSectorList> sectors = app->GetEngine()->GetSectors();
  float x = app->GetCamera()->GetTransform ().GetOrigin ()[0];
  float y = app->GetCamera()->GetTransform ().GetOrigin ()[2];
  int sectorX = int((x+5)/10);
  int sectorY = int((y+5)/10);
  char* str = new char [30];
  sprintf(str, "room_%i_0_%i", sectorX,sectorY );     
  csRef<iSector> newUserSector = sectors->FindByName(str);
  user->GetMovable()->SetSector(newUserSector);
  csVector3 userPosition = app->GetCamera()->GetTransform().GetOrigin();
  //The sprite sits below the camera.
  userPosition = userPosition + csVector3(0,-3,0);
  user->GetMovable()->SetPosition(userPosition);
}

void Player::StartRotation (const csVector3& rot)
{
  float angle = ROTATE_ANGLE;
  csOrthoTransform trans = app->GetCamera ()->GetTransform ();
  start_lookat = trans.This2OtherRelative (csVector3 (0, 0, 1));
  trans.RotateThis (rot, angle);
  desired_lookat = trans.This2OtherRelative (csVector3 (0, 0, 1));
  amount_rotated = 0;
}

//-----------------------------------------------------------------------------

Explosion::Explosion (iMeshWrapper* mesh, int timeleft)
{
  Explosion::mesh = mesh;
  Explosion::timeleft = timeleft;
}

bool Explosion::Handle (csTicks elapsed_ticks)
{
  timeleft -= elapsed_ticks;
  if (timeleft <= 0)
  {
    return false;
  }
  return true;
}

//-----------------------------------------------------------------------------

Adversary::Adversary (AppMazing* app,
	iMeshWrapper* mesh, const RoomCoordinate& rc) :
	scfImplementationType (this)
{
  Adversary::app = app;
  Adversary::mesh = mesh;
  current_location = rc;
  moving = false;
}

void Adversary::ThinkAndMove (float elapsed_seconds)
{
  if (!moving)
  {
    Maze* maze = app->GetGame ().GetMaze ();
    const csArray<RoomCoordinate>& connections = maze->GetConnections (
	current_location);
    size_t moveto = (rand () >> 3) % connections.GetSize ();
    if (maze->IsSpaceFree (connections[moveto]))
    {
      start.x = float (current_location.x) * ROOM_DIMENSION;
      start.y = float (current_location.y) * ROOM_DIMENSION;
      start.z = float (current_location.z) * ROOM_DIMENSION;
      maze->FreeSpace (current_location);
      current_location = connections[moveto];
      maze->OccupySpace (current_location);
      end.x = float (current_location.x) * ROOM_DIMENSION;
      end.y = float (current_location.y) * ROOM_DIMENSION;
      end.z = float (current_location.z) * ROOM_DIMENSION;
      remaining_seconds = ADVERSARY_MOVETIME;
      moving = true;
    }
  }
  else
  {
    remaining_seconds -= elapsed_seconds;
    csVector3 new_pos;
    if (remaining_seconds <= 0)
    {
      moving = false;
      new_pos = end;
    }
    else
    {
      csMath3::Between (end, start, new_pos,
      	100.0 * remaining_seconds / ADVERSARY_MOVETIME, 0);
    }
    iMovable* movable = mesh->GetMovable ();
    iSector* old_sector = movable->GetSectors ()->Get (0);
    bool mirror;
    iSector* new_sector = old_sector->FollowSegment (
    	movable->GetTransform (), new_pos, mirror, true);
    movable->SetSector (new_sector);
    movable->GetTransform ().SetOrigin (new_pos);
    movable->UpdateMove ();
  }
}

//-----------------------------------------------------------------------------

Laser::Laser (AppMazing* app)
{
  Laser::app = app;
  lasertime = 0;
}

void Laser::Start ()
{
  if (lasertime > 0) return;	// Laser already in progress.
  lasertime = LASER_LIFETIME;
  const csOrthoTransform& tr = app->GetCamera ()->GetTransform ();
  laserstart = tr.This2Other (LASER_OFFSET);
  laserend = tr.This2Other (csVector3 (0, 0, 20.0));
  lasersector = app->GetCamera ()->GetSector ();

  // Fire up our beam.
  laserbeam->GetMovable ()->SetPosition (lasersector, laserstart);
  laserbeam->GetMovable ()->GetTransform ().LookAt (
  	laserend-tr.GetOrigin (), csVector3 (0, 1, 0));
  laserbeam->GetMovable ()->UpdateMove ();
}

void Laser::Handle (csTicks ticks)
{
  if (lasertime <= 0) return;
  lasertime -= ticks;
  if (lasertime <= 0)
  {
    // Time to stop the laser.
    lasertime = 0;
    laserbeam->GetMovable ()->ClearSectors ();
    laserbeam->GetMovable ()->UpdateMove ();
  }
  else
  {
    int flick = (lasertime / LASER_FLICKTIME) & 1;
    Check ();
    laserbeam->GetMeshObject ()->SetColor (
      flick ? csColor (2.0, 2.0, 2.0) : csColor (.5, .5, .5));
  }
}

void Laser::Check ()
{
  // Do a beam to check if we hit an adversary.
  csSectorHitBeamResult rc = lasersector->HitBeamPortals (
  	laserstart, laserend);
  if (rc.mesh)
  {
    csRef<Adversary> adv = CS_GET_CHILD_OBJECT (
    	rc.mesh->QueryObject (), Adversary);
    if (adv)
    {
      // Hit!
      app->GetGame ().ExplodeAdversary (adv);
    }
  }
}

//-----------------------------------------------------------------------------

Game::Game (AppMazing* app) :
  	app (app),
	player (app),
	maze (app),
	laser (app)
{
}

bool Game::CreateFactories ()
{
  csRef<iGeneralFactoryState> fstate;
  iEngine* engine = app->GetEngine ();
  iLoader* loader = app->GetLoader ();

  //---------------------------------------------------------------------
  // Adversary factory.
  adversary_factory = engine->CreateMeshFactory (
  	"crystalspace.mesh.object.genmesh", "adversary");
  if (!adversary_factory) return false;

  fstate = scfQueryInterface<iGeneralFactoryState> (
  	adversary_factory->GetMeshObjectFactory ());
  csEllipsoid ellips (
  	csVector3 (0, 0, 0),
	csVector3 (ADVERSARY_DIMENSION, ADVERSARY_DIMENSION,
		ADVERSARY_DIMENSION));
  fstate->GenerateSphere (ellips, 10);

  if (!loader->LoadTexture ("adversary_texture", "/lib/stdtex/misty.jpg"))
    return app->ReportError ("Error loading 'misty' texture!");
  iMaterialWrapper* adversary_material = engine->GetMaterialList ()
  	->FindByName ("adversary_texture");
  adversary_factory->GetMeshObjectFactory ()
    ->SetMaterialWrapper (adversary_material);

  //---------------------------------------------------------------------
  // Beam factory.
  laserbeam_factory = engine->CreateMeshFactory (
  	"crystalspace.mesh.object.genmesh", "laserbeam");
  if (!laserbeam_factory) return false;

  fstate = scfQueryInterface<iGeneralFactoryState> (
  	laserbeam_factory->GetMeshObjectFactory ());
  csBox3 laser_box (
  	csVector3 (-LASER_WIDTH, -LASER_WIDTH, 0),
  	csVector3 (LASER_WIDTH, LASER_WIDTH, LASER_LENGTH));
  fstate->GenerateBox (laser_box);
  fstate->SetLighting (false);
  fstate->SetColor (csColor (1.0, 1.0, 1.0));
  // We don't want to hit the player against the laserbeam when it is
  // visible so we disable the collision detection mesh here.
  csStringID colldet_id = app->GetStrings ()->Request ("colldet");
  laserbeam_factory->GetMeshObjectFactory ()->GetObjectModel ()
  	->SetTriangleData (colldet_id, 0);

  if (!loader->LoadTexture ("laserbeam_texture", "/lib/stdtex/blobby.jpg"))
    return app->ReportError ("Error loading 'blobby' texture!");
  iMaterialWrapper* laserbeam_material = engine->GetMaterialList ()
  	->FindByName ("laserbeam_texture");
  laserbeam_factory->GetMeshObjectFactory ()
    ->SetMaterialWrapper (laserbeam_material);

  //---------------------------------------------------------------------
  // Beam object.
  csRef<iMeshWrapper> laserbeam = engine->CreateMeshWrapper (
  	laserbeam_factory, "laserbeam");
  if (!laserbeam)
    return app->ReportError ("Error creating laserbeam mesh!");
  // Set our laser beam to NOHITBEAM so that we can use HitBeam() methods
  // to find out what our laser hits without HitBeam() returning the
  // laser itself.
  laserbeam->GetFlags ().Set (CS_ENTITY_NOHITBEAM);
  laser.SetMeshWrapper (laserbeam);

  //---------------------------------------------------------------------
  // Explosion factory.
  explosion_factory = engine->CreateMeshFactory (
  	"crystalspace.mesh.object.particles", "explosion");
  if (!explosion_factory) return false;

  csRef<iParticleSystemFactory> pbase = scfQueryInterface<
  	iParticleSystemFactory> (explosion_factory->GetMeshObjectFactory ());

  if (!loader->LoadTexture ("explosion_texture", "/lib/std/spark.png"))
    return app->ReportError ("Error loading 'spark' texture!");
  iMaterialWrapper* explosion_material = engine->GetMaterialList ()
  	->FindByName ("explosion_texture");
  
  explosion_factory->GetMeshObjectFactory ()->SetMaterialWrapper (explosion_material);
  explosion_factory->GetMeshObjectFactory ()->SetMixMode (CS_FX_ADD);

  pbase->SetParticleSize (csVector2 (0.2f, 0.2f));
  pbase->SetParticleRenderOrientation (CS_PARTICLE_CAMERAFACE_APPROX);
//  pbase->SetDeepCreation (true);

  csRef<iParticleBuiltinEmitterFactory> emitter_factory = 
    csLoadPluginCheck<iParticleBuiltinEmitterFactory> (app->GetObjectRegistry (),
    "crystalspace.mesh.object.particles.emitter");
  
  csRef<iParticleBuiltinEmitterSphere> emitter = emitter_factory->CreateSphere ();
  emitter->SetRadius (0);
  emitter->SetParticlePlacement (CS_PARTICLE_BUILTIN_CENTER);
  emitter->SetDuration (EXPLOSION_EMITTIME / 1000.0f);
  emitter->SetInitialTTL (EXPLOSION_PARTTIMELOW / 1000.0f, EXPLOSION_PARTTIMEHIGH / 1000.0f);
  emitter->SetUniformVelocity (false);
  emitter->SetInitialVelocity (csVector3 (3.0f,0,0), csVector3 (0.0f));
  emitter->SetEmissionRate (50);

  pbase->AddEmitter (emitter);

  return true;
}

bool Game::CreateAdversary (int x, int y, int z)
{
  float sx = float (x) * ROOM_DIMENSION;
  float sy = float (y) * ROOM_DIMENSION;
  float sz = float (z) * ROOM_DIMENSION;
  csRef<iMeshWrapper> adversary = app->GetEngine ()->CreateMeshWrapper (
      	adversary_factory, "adversary",
	maze.GetSector (x, y, z), csVector3 (sx, sy, sz));
  if (!adversary)
    return app->ReportError ("Couldn't create adversary mesh!");

  RoomCoordinate rc (x, y, z);
  Adversary* adv = new Adversary (app, adversary, rc);
  adversaries.Push (adv);
  adversary->QueryObject ()->ObjAdd ((iObject*)adv);
  adv->DecRef ();
  
  return true;
}

bool Game::CreateSprites2 ()
{
  //setup material
  iTextureWrapper* textureBarV = app->GetLoader()->LoadTexture ("barv",
    "/lib/std/barv.gif");
  if (textureBarV == 0)
    return app->ReportError("Error loading barv texture!");
   
  iTextureWrapper* textureBarV3 = app->GetLoader()->LoadTexture ("barv3",
    "/lib/std/barv3.gif");
  if (textureBarV3 == 0)
    return app->ReportError("Error loading barv3 texture!");
  iMaterialWrapper* materialBarV3= app->GetEngine()->
    CreateMaterial("MatBarV3",textureBarV3);

  //setup rooms
  csRef<iSectorList> sectors = app->GetEngine()->GetSectors();
  csRef<iSector> room = sectors->FindByName("room_1_0_1");

  csRef<iMeshWrapper> cubeMeshWrapper = app->GetEngine()->
    CreateThingMesh (room, "cube");
  if (cubeMeshWrapper == 0)
    app->ReportError("Error createing cube mesh wrapper!");
  iMeshObject* cubeObject = cubeMeshWrapper->GetMeshObject();
  if (cubeObject == 0)
    app->ReportError("Error getting cube object");

  csRef<iThingFactoryState> cubeFactoryState =  
    scfQueryInterface<iThingFactoryState> (cubeObject->GetFactory());
  if (cubeFactoryState == 0)
    app->ReportError("Error getting factory state");
  float xoffset= 10.0;
  float yoffset= 10.0;
  float zoffset= -0.5;

  float p1x = 0.0;  float p1y = 0.0; float p1z = 0.0;
  float p2x = 0.0;  float p2y = 2.0; float p2z = 0.0;
  float p3x = 2.0;  float p3y = 2.0; float p3z = 0.0;
  float p4x = 2.0;  float p4y = 0.0; float p4z = 0.0;
  float p5x = 1.0;  float p5y = 1.0; float p5z = 1.4;
  cubeFactoryState->AddQuad ( 
     csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
     csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y), 
     csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
     csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y));
  cubeFactoryState->AddTriangle ( //left
     csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
     csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
     csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y));
  cubeFactoryState->AddTriangle ( //front
     csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
     csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
     csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));
  cubeFactoryState->AddTriangle ( //right
     csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
     csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
     csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));
  cubeFactoryState->AddTriangle ( //back
     csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
     csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
     csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));

  iMaterialWrapper* materialBarV1= app->GetEngine()->
    CreateMaterial("MatBarV1",textureBarV);
  csPolygonRange allPolygons = csPolygonRange(0,1);
  cubeFactoryState->SetPolygonMaterial (allPolygons, materialBarV1);
  cubeFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);

  allPolygons = csPolygonRange(2,5);
  cubeFactoryState->SetPolygonMaterial (allPolygons, materialBarV3);
  cubeFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);

  return true;

}

bool Game::CreateHStripedTriangle(int shape,int xRoom,int yRoom,float zoffset)
  {
  float xoffset = xRoom*10.0;
  float yoffset = yRoom*10.0;

  char* str = new char [30];
  sprintf(str, "room_%i_0_%i", xRoom,yRoom );     
  csRef<iSectorList> sectors = app->GetEngine()->GetSectors();
  csRef<iSector> triangleSector = sectors->FindByName(str);
  csRef<iMeshWrapper> triangleMeshWrapper = app->GetEngine()->
    CreateThingMesh (triangleSector, "triangle");
  if (triangleMeshWrapper == 0)
    app->ReportError("Error createing triangle mesh wrapper!");
  iMeshObject* triangleObject = triangleMeshWrapper->GetMeshObject();
  if (triangleObject == 0)
    app->ReportError("Error getting triangle object");
  csRef<iThingFactoryState> triangleFactoryState =  
    scfQueryInterface<iThingFactoryState> (triangleObject->GetFactory());
  if (triangleFactoryState == 0) 
    app->ReportError("Error getting factory state");

  //I had a problem with the bars not wrapping around correctly.  
  //My fix is that each triangle consists of two parts: triangle and rest

  csRef<iMeshWrapper> restMeshWrapper = app->GetEngine()->
    CreateThingMesh (triangleSector, "rest");
  if (restMeshWrapper == 0)
    app->ReportError("Error createing rest mesh wrapper!");
  iMeshObject* restObject = restMeshWrapper->GetMeshObject();
  if (restObject == 0)
    app->ReportError("Error getting rest object");
  csRef<iThingFactoryState> restFactoryState =  
    scfQueryInterface<iThingFactoryState> (restObject->GetFactory());
  if (restFactoryState == 0) 
    app->ReportError("Error getting rest factory state");

  float p1x = 0.0;  float p1y = 0.0; 
  float p2x = 0.0;  float p2y = 2.0; 
  float p3x = 2.0;  float p3y = 2.0; 
  float p4x = 2.0;  float p4y = 0.0; 
  float p5x = 1.0;  float p5y = 1.0; 
  if (shape == 0) {
    float p1z = 0.0;
    float p2z = 0.0;
    float p3z = 0.0;
    float p4z = 0.0;
    float p5z = 1.4;
    triangleFactoryState->AddQuad ( 
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y), 
       csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
       csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y));
    triangleFactoryState->AddTriangle ( //left
      csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
      csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
      csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y));
    triangleFactoryState->AddTriangle ( //right
      csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
      csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
      csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));
    restFactoryState->AddTriangle ( //front
      csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
      csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
      csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));
    restFactoryState->AddTriangle ( //back
      csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
      csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
      csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));
  }
  else if (shape == 1) {
    float p1z = 1.4;
    float p2z = 1.4;
    float p3z = 1.4;
    float p4z = 1.4;
    float p5z = 0.0;
    triangleFactoryState->AddQuad ( 
       csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
       csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y), 
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y));
    restFactoryState->AddTriangle ( //left
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
       csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
       csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y));
    triangleFactoryState->AddTriangle ( //front
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
       csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y));
    restFactoryState->AddTriangle ( //right
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y));
    triangleFactoryState->AddTriangle ( //back
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
       csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y));
  }
  else  return app->ReportError("Error Bad Create Triangle Shape!");

  iTextureWrapper* textureBarV = app->GetLoader()->LoadTexture ("barv",
    "/lib/std/barv.gif");
  if (textureBarV == 0)
    return app->ReportError("Error loading barv texture!");

  iTextureWrapper* textureBarH = app->GetLoader()->LoadTexture ("barh",
    "/lib/std/barh.gif");
  if (textureBarH == 0)
    return app->ReportError("Error loading barh texture!");

  iMaterialWrapper* materialBarH= app->GetEngine()->
    CreateMaterial("MatBarH",textureBarH);


  csPolygonRange allPolygons = csPolygonRange(0,2);
  triangleFactoryState->SetPolygonMaterial (allPolygons, materialBarH);
  triangleFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);

  allPolygons = csPolygonRange(0,1);
  restFactoryState->SetPolygonMaterial (allPolygons, materialBarH);
  restFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);

  return true;
}

bool Game::CreateVStripedPyramid(int xRoom, int yRoom,   float zoffset) {
  float xoffset = xRoom*10.0;
  float yoffset = yRoom*10.0;

  char* str = new char [30];
  sprintf(str, "room_%i_0_%i", xRoom,yRoom );     
  csRef<iSectorList> sectors = app->GetEngine()->GetSectors();
  csRef<iSector> triangleSector = sectors->FindByName(str);
  csRef<iMeshWrapper> triangleMeshWrapper = app->GetEngine()->
    CreateThingMesh (triangleSector, "triangle");
  if (triangleMeshWrapper == 0)
    app->ReportError("Error createing triangle mesh wrapper!");
  iMeshObject* triangleObject = triangleMeshWrapper->GetMeshObject();
  if (triangleObject == 0)
    app->ReportError("Error getting triangle object");

  csRef<iThingFactoryState> triangleFactoryState =  
    scfQueryInterface<iThingFactoryState> (triangleObject->GetFactory());
  if (triangleFactoryState == 0) 
    app->ReportError("Error getting factory state");

  float p1x = 0.0;  float p1y = 1.4; 
  float p2x = 1.4;  float p2y = 2.8; 
  float p3x = 2.8;  float p3y = 1.4; 
  float p4x = 1.4;  float p4y = 0.0; 
  float p5x = 1.4;  float p5y = 1.4; 
  float p1z = 0.0;
  float p2z = 0.0;
  float p3z = 0.0;
  float p4z = 0.0;
  float p5z = 1.4;
  triangleFactoryState->AddQuad ( 
    csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
    csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y), 
    csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
    csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y));
  triangleFactoryState->AddTriangle ( //left
    csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
    csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
    csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y));
  triangleFactoryState->AddTriangle ( //back
    csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
    csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
    csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));

  iTextureWrapper* textureBarL = app->GetLoader()->LoadTexture ("barl",
    "/lib/std/leftslant.gif");
  //"/lib/std/asphalt.jpg");
  if (textureBarL == 0)
    return app->ReportError("Error loading barl texture!");

  iMaterialWrapper* materialBarL= app->GetEngine()->
    CreateMaterial("MatBarL",textureBarL);

  csPolygonRange allPolygons = csPolygonRange(0,2);
  triangleFactoryState->SetPolygonMaterial (allPolygons, materialBarL);
  triangleFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);


  csRef<iMeshWrapper> sideMeshWrapper = app->GetEngine()->
    CreateThingMesh (triangleSector, "side");
  if (sideMeshWrapper == 0)
    app->ReportError("Error createing side mesh wrapper!");
  iMeshObject* sideObject = sideMeshWrapper->GetMeshObject();
  if (sideObject == 0)
    app->ReportError("Error getting side object");

  csRef<iThingFactoryState> sideFactoryState =  
    scfQueryInterface<iThingFactoryState> (sideObject->GetFactory());
  if (sideFactoryState == 0) 
    app->ReportError("Error getting side factory state");

  sideFactoryState->AddTriangle ( //front
       csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));
  sideFactoryState->AddTriangle ( //right
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y));

  iTextureWrapper* textureBarR = app->GetLoader()->LoadTexture ("barr",
    "/lib/std/rightslant.gif");
  if (textureBarR == 0)
    return app->ReportError("Error loading barr texture!");
  iMaterialWrapper* materialBarR= app->GetEngine()->
    CreateMaterial("MatBarR",textureBarR);

    //  allPolygons = csPolygonRange(0,1);
  allPolygons = csPolygonRange(0,1);
  sideFactoryState->SetPolygonMaterial (allPolygons, materialBarR);
  sideFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);

  return true;
}

bool Game::CreateVStripedStalactite(int xRoom, int yRoom,   float zoffset) {
  float xoffset = xRoom*10.0;
  float yoffset = yRoom*10.0;

  char* str = new char [30];
  sprintf(str, "room_%i_0_%i", xRoom,yRoom );     
  csRef<iSectorList> sectors = app->GetEngine()->GetSectors();
  csRef<iSector> triangleSector = sectors->FindByName(str);
  csRef<iMeshWrapper> triangleMeshWrapper = app->GetEngine()->
    CreateThingMesh (triangleSector, "triangle");
  if (triangleMeshWrapper == 0)
    app->ReportError("Error createing triangle mesh wrapper!");
  iMeshObject* triangleObject = triangleMeshWrapper->GetMeshObject();
  if (triangleObject == 0)
    app->ReportError("Error getting triangle object");

  csRef<iThingFactoryState> triangleFactoryState =  
    scfQueryInterface<iThingFactoryState> (triangleObject->GetFactory());
  if (triangleFactoryState == 0) 
    app->ReportError("Error getting factory state");

  float p1x = 0.0;  float p1y = 1.4; 
  float p2x = 1.4;  float p2y = 2.8; 
  float p3x = 2.8;  float p3y = 1.4; 
  float p4x = 1.4;  float p4y = 0.0; 
  float p5x = 1.4;  float p5y = 1.4; 
  float p1z = 1.4;
  float p2z = 1.4;
  float p3z = 1.4;
  float p4z = 1.4;
  float p5z = 0.0;
    triangleFactoryState->AddQuad ( 
       csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
       csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y), 
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y));
    triangleFactoryState->AddTriangle ( //front
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y),
       csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y));
    triangleFactoryState->AddTriangle ( //right
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y),
       csVector3 (xoffset+p3x, zoffset+p3z, yoffset+p3y));

  iTextureWrapper* textureBarL = app->GetLoader()->LoadTexture ("barl",
    "/lib/std/leftslant.gif");
  //"/lib/std/asphalt.jpg");
  if (textureBarL == 0)
    return app->ReportError("Error loading barl texture!");

  iMaterialWrapper* materialBarL= app->GetEngine()->
    CreateMaterial("MatBarL",textureBarL);

  csPolygonRange allPolygons = csPolygonRange(0,2);
  triangleFactoryState->SetPolygonMaterial (allPolygons, materialBarL);
  triangleFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);


  csRef<iMeshWrapper> sideMeshWrapper = app->GetEngine()->
    CreateThingMesh (triangleSector, "side");
  if (sideMeshWrapper == 0)
    app->ReportError("Error createing side mesh wrapper!");
  iMeshObject* sideObject = sideMeshWrapper->GetMeshObject();
  if (sideObject == 0)
    app->ReportError("Error getting side object");

  csRef<iThingFactoryState> sideFactoryState =  
    scfQueryInterface<iThingFactoryState> (sideObject->GetFactory());
  if (sideFactoryState == 0) 
    app->ReportError("Error getting side factory state");

  sideFactoryState->AddTriangle ( //left
    csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
    csVector3 (xoffset+p2x, zoffset+p2z, yoffset+p2y),
     csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y));
  sideFactoryState->AddTriangle ( //back
       csVector3 (xoffset+p5x, zoffset+p5z, yoffset+p5y),
       csVector3 (xoffset+p1x, zoffset+p1z, yoffset+p1y),
       csVector3 (xoffset+p4x, zoffset+p4z, yoffset+p4y));

  iTextureWrapper* textureBarR = app->GetLoader()->LoadTexture ("barr",
    "/lib/std/rightslant.gif");
  if (textureBarR == 0)
    return app->ReportError("Error loading barr texture!");
  iMaterialWrapper* materialBarR= app->GetEngine()->
    CreateMaterial("MatBarR",textureBarR);

    //  allPolygons = csPolygonRange(0,1);
  allPolygons = csPolygonRange(0,1);
  sideFactoryState->SetPolygonMaterial (allPolygons, materialBarR);
  sideFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);

  return true;
}

//create the stalactites and pyramids.  Largely taken from simple2.
bool Game::CreateSprites ()
{ /* //vpyr, hpyr, hstal, vstal
  if (!CreateVStripedStalactite(5,1,0.5)) return false;
  if (!CreateHStripedTriangle(0,1,5,-0.5)) return false;
  if (!CreateVStripedPyramid(1,1,-0.5)) return false;
  if (!CreateHStripedTriangle(1,5,5,0.5)) return false;
  */
  //hpyr,vpyr,vstal,hstal
  if (!CreateHStripedTriangle(0,1,1,-0.5)) return false;
  if (!CreateVStripedStalactite(5,5,0.5)) return false;
  if (!CreateVStripedPyramid(1,5,-0.5)) return false;
  if (!CreateHStripedTriangle(1,5,1,0.5)) return false;

  /*
  cubeFactoryState->AddQuad ( //top
     csVector3 (-.5, .5, -.5),
     csVector3 (.5, .5, -.5),
     csVector3 (.5, .5, .5),
     csVector3 (-.5, .5, .5));
  cubeFactoryState->AddQuad ( //bottom
     csVector3 (-.5, .5, -.5),
     csVector3 (-.5, .5, .5),
     csVector3 (.5, .5, .5),
     csVector3 (.5, .5, -.5));
  cubeFactoryState->AddQuad (//front
     csVector3 (-.5, -.5, .5),
     csVector3 (.5, -.5, .5),
     csVector3 (.5, .5, .5),
     csVector3 (-.5, .5, .5));
  cubeFactoryState->AddQuad (//back
     csVector3 (-.5, -.5, -.5),
     csVector3 (.5, -.5, -.5),
     csVector3 (.5, .5, -.5),
     csVector3 (-.5, .5, -.5));
  cubeFactoryState->AddQuad (//left
     csVector3 (-.5, -.5, -.5),
     csVector3 (-.5, .5, -.5),
     csVector3 (-.5, .5, .5),
     csVector3 (-.5, -.5, .5));
  cubeFactoryState->AddQuad (//right
     csVector3 (.5, -.5, -.5),
     csVector3 (.5, .5, -.5),
     csVector3 (.5, .5, .5),
     csVector3 (.5, -.5, .5));

  csPolygonRange allPolygons = csPolygonRange(0,4);
  iMaterialWrapper* materialBarV= app->GetEngine()->
  CreateMaterial("MatBarV",textureBarV);
  cubeFactoryState->SetPolygonMaterial (allPolygons, materialBarV);
  iTextureWrapper* textureColors = app->GetLoader()->LoadTexture ("colors",
      "/lib/std/colors.jpg");
  iMaterialWrapper* materialColor= app->GetEngine()->
    CreateMaterial("MatColor",textureColors);
  cubeFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);
  allPolygons = csPolygonRange(5,5);
  cubeFactoryState->SetPolygonMaterial (allPolygons, materialColor);
  cubeFactoryState->SetPolygonTextureMappingEnabled (allPolygons, false);
  */

  return true;
}

bool Game::CreateAgents () {
  //bool movingAgent = false;
  //load the texture
  iTextureWrapper* textureSpark = app->GetLoader()->LoadTexture ("spark",
    "/lib/std/spark.png");
  if (textureSpark == 0)
    return app->ReportError("Error loading barh texture!");


  // Load a sprite template from disk.
  csRef<iMeshFactoryWrapper> imeshfact (
    app->GetLoader()->LoadMeshObjectFactory ("/lib/std/sprite1"));
  if (imeshfact == 0)
    app->ReportError("Error loading agent object factory!");

  //agentLocation.Set(20,0,20);
  agentLocation.Set(20,0,11);
  // Create the sprite and add it to the engine.
  csRef<iSectorList> sectors = app->GetEngine()->GetSectors();
  csRef<iSector> testSectorAgent = sectors->FindByName("room_2_0_1");
  csVector3 agentSpriteLocation = agentLocation + csVector3(0,-2,0);
  csRef<iMeshWrapper> spriteAgent (app->GetEngine()->CreateMeshWrapper (
    imeshfact, "SpriteAgent", testSectorAgent,agentSpriteLocation));
  //    csVector3 (agentStartX, agentStartHeight, agentStartY)));

  csMatrix3 m; 
  m.Identity ();
  spriteAgent->GetMovable ()->SetTransform (m);
  spriteAgent->GetMovable ()->UpdateMove ();
  csRef<iSprite3DState> spstate = scfQueryInterface<iSprite3DState> 
    (spriteAgent->GetMeshObject ());
  spstate->SetAction ("default");

  agent = spriteAgent;
  //agent->GetMovable()->SetTransform(ot);
  agent->GetMovable()->UpdateMove();

  //create the agent view
  csRef<iGraphics3D> g3d = csQueryRegistry<iGraphics3D>
   (app->GetObjectRegistry());
  agentView.AttachNew(new csView (app->GetEngine(), g3d));

  // Now we need to position the camera in our world.
  agentView->GetCamera ()->SetSector (testSectorAgent);
  agentView->GetCamera ()->GetTransform ().SetOrigin (agentLocation);
  //(csVector3 (agentStartX, agentStartHeight, agentStartY));

  //agentLookat.Set(-1, 0, -1);
  agentLookat.Set(-1, 0, 0);
  iCamera* agentCamera = agentView->GetCamera();
  
  agentCamera->GetTransform ().LookAt (agentLookat, csVector3 (0, 1, 0));
  csReversibleTransform trans = agentView->GetCamera()->GetTransform ();
  //  csVector3 move_dist = csVector3(-2, 0, -2);
  //  agentView->GetCamera()->MoveWorld(move_dist, false);

  ///USER
  //make the sprite for the user
  csRef<iMeshFactoryWrapper> imeshfactUser (
    app->GetLoader()->LoadMeshObjectFactory ("/lib/std/sprite2"));
  if (imeshfactUser == 0)
    app->ReportError("Error loading agent object factory!");

  // Create the sprite and add it to the engine.
  csRef<iSector> testSectorUser = sectors->FindByName("room_0_0_0");
  csVector3 userLocation = csVector3(10,-3,10); //get it from the camera
  csRef<iMeshWrapper> userSprite (app->GetEngine()->CreateMeshWrapper (
    imeshfactUser, "UserSprite", testSectorUser,userLocation));

  user=userSprite;

  return true;
}

void Game::ExecuteAgentCommand(int command) {
  if (command == 1) { //left
    TurnAgent(1); }
  else if (command == 2) { //right
    TurnAgent(2);}
  else if (command == 3) { //forward
    MoveAgent(1);}
  else if (command == 4) { //back
    MoveAgent(-1);}
  else
    app->ReportError("Bad ExecuteAgentCommand");

}

//modified from Player::StartMovement
//return true if moving the agent will collide with something, false otherwise.
bool Game::willAgentCollide(csVector3 move_dist) {

  // First we find all meshes that are near the path that we want to move.
  // To do that we take the center point of our movement vector and
  // calculate all objects that touch the circle around that point with
  // radius (MOVE_DISTANCE+PLAYER_SIZE) / 2.0.
  csRef<iMeshWrapperIterator> it = app->GetEngine ()->GetNearbyMeshes (
    agentView->GetCamera()->GetSector (), 
    (agent->GetMovable()->GetFullPosition()),
	(PLAYER_SIZE));

  if (!it->HasNext ())
  {
    // We have no objects in the iterator so there can be no collision.
    // We can move freely.
    return false;
  }

  // If we have meshes then we will calculate collision detection for our
  // object along the path we want to move.
  iCollideSystem* cdsys = app->GetCollisionDetectionSystem ();
  csVector3 newAgentLocation = agent->GetMovable()->GetFullPosition()+
    move_dist;

  it->Reset ();
  while (it->HasNext ()) {
    iMeshWrapper* mesh = it->Next ();
    csColliderWrapper* collide_wrap = csColliderWrapper::GetColliderWrapper (
      mesh->QueryObject ());
    if (collide_wrap) {
      csReversibleTransform mesh_trans = mesh->GetMovable ()
        ->GetFullTransform ();
      csReversibleTransform camera_trans=agentView->GetCamera()->
        GetTransform();
      camera_trans.SetOrigin (newAgentLocation);

      cdsys->ResetCollisionPairs ();
      if (cdsys->Collide (
      	      collide_wrap->GetCollider (), &mesh_trans,
	      agent_collider, &camera_trans))
        {
        // Collision, so we can stop. 
	return true;
        }
      }
  }

  return false;
}

void Game::MoveAgent(int dist) {
  csReversibleTransform trans = agentView->GetCamera()->GetTransform ();
  csVector3 move_dist = agentLookat * dist;

  if (willAgentCollide(move_dist)) 
    {
      /*
    app->ReportError ("Agent Collide!"); //undone
    app->ReportError (agentView->GetCamera()->GetSector()->QueryObject()->GetName()); //undone
    char* str = new char[30]; 
    float flt = agent->GetMovable()->GetFullPosition()[0];  
    sprintf(str, "%.4g", flt ); 
    app->ReportError (str); 
      */
    userCommand.Clear();
    userCommand.Append("*Bump*");
    sendAgentCommandToCANT();
    userCommand.Clear();

    return;
    }

  agentView->GetCamera()->MoveWorld(move_dist, false);

  agent->GetMovable()->MovePosition(move_dist);
  //set the sector
  float x = agentView->GetCamera()->GetTransform ().GetOrigin ()[2];
  float y = agentView->GetCamera()->GetTransform ().GetOrigin ()[2];
  int sectorX = int((x+5)/10);
  int sectorY = int((y+5)/10);
  char* str = new char [30];
  sprintf(str, "room_%i_0_%i", sectorX,sectorY );     
  csRef<iSectorList> sectors = app->GetEngine()->GetSectors();
  csRef<iSector> newAgentSector = sectors->FindByName(str);
  agent->GetMovable()->SetSector(newAgentSector);
}

void Game::TurnAgent(int direction) {
  if (direction == 1) { //left
    float angle = -0.1;
    csOrthoTransform trans = agentView->GetCamera ()->GetTransform ();
    trans.RotateThis (csVector3(0,1,0), angle);
    agentLookat = trans.This2OtherRelative (csVector3 (0, 0, 1));
    agentView->GetCamera ()->GetTransform ().LookAt(agentLookat,csVector3(0,1,0));
  }
  else if (direction == 2) { //right
    float angle = 0.1;
    csOrthoTransform trans = agentView->GetCamera ()->GetTransform ();
    trans.RotateThis (csVector3(0,1,0), angle);
    agentLookat = trans.This2OtherRelative (csVector3 (0, 0, 1));
    agentView->GetCamera ()->GetTransform ().LookAt(agentLookat,csVector3(0,1,0));
  }
}



bool Game::InitCollisionDetection ()
{
  csColliderHelper::InitializeCollisionWrappers (
      app->GetCollisionDetectionSystem (), app->GetEngine (), 0);
  return player.InitCollisionDetection ();
}

bool Game::SetupGame ()
{
  if (!maze.CreateGeometry ())
    return app->ReportError("Error creating the geometry!");

  if (!CreateFactories ())
    return app->ReportError ("Error creating mesh factories!");

  iEngine* engine = app->GetEngine ();
  engine->Prepare ();


  if (!InitCollisionDetection ())
    return false;

  if (!InitAgentCollisionDetection ())
    return false;

  /*
  if (!CreateAdversary (0, 0, 2)) return false;
  if (!CreateAdversary (1, 0, 2)) return false;
  */

  if(! CreateSprites()) return false;
  if(! CreateAgents()) return false;

  return true;
}

void Game::ExplodeAdversary (Adversary* adv)
{
  iMeshWrapper* mesh = adv->GetMesh ();
  StartExplosion (mesh->GetMovable ()->GetSectors ()->Get (0),
      	mesh->GetMovable ()->GetTransform ().GetOrigin ());
  app->GetEngine ()->RemoveObject (mesh);
  adversaries.Delete (adv);
}

void Game::StartExplosion (iSector* sector, const csVector3& pos)
{
  csRef<iMeshWrapper> explo = app->GetEngine ()->CreateMeshWrapper (
      explosion_factory, "explosion", sector, pos);
  if (!explo)
  {
    app->ReportError ("Error creating explosion mesh!");
    return;
  }
  explo->SetZBufMode (CS_ZBUF_TEST);
  Explosion exp (explo, EXPLOSION_TIME);
  explosions.Push (exp);
}

void Game::HandleExplosions (csTicks elapsed_ticks)
{
  size_t i = 0;
  while (i < explosions.GetSize ())
  {
    if (explosions[i].Handle (elapsed_ticks)) i++;
    else
    {
      app->GetEngine ()->RemoveObject (explosions[i].GetMesh ());
      explosions.DeleteIndex (i);
    }
  }
}

void Game::Handle (csTicks elapsed_ticks)
{
  float elapsed_seconds = float (elapsed_ticks) / 1000.0;

  // Handle the laser.
  laser.Handle (elapsed_ticks);

  // Handle explosions.
  HandleExplosions (elapsed_ticks);

  // Move the camera.
  player.MoveAndRotateCamera (elapsed_seconds);

  //Dump screens if necessary
  csRef<iVirtualClock> vc = csQueryRegistry<iVirtualClock>(app->
    GetObjectRegistry());
  PrintPictureOccasionally(vc->GetCurrentTicks ());

  //See if CABot has asked us to do anything and do it
  ProcessAgentCommand();

  // Let all the adversaries think about what to do.
  size_t i;
  for (i = 0 ; i < adversaries.GetSize () ; i++)
    adversaries[i]->ThinkAndMove (elapsed_seconds);
}

bool Game::OnKeyboard(iEvent& ev)
{
  // We got a keyboard event.
  if (csKeyEventHelper::GetEventType(&ev) == csKeyEventTypeDown)
  {
    // The user pressed a key (as opposed to releasing it).
    utf32_char code = csKeyEventHelper::GetCookedCode(&ev);
    switch (code)
    {
      //case CSKEY_F2:  display agent camera in appmazing
      case CSKEY_F3:
	PrintPicture ();
	return true;
      case CSKEY_F4:
	laser.Start ();
	return true;
      case CSKEY_F5:
        {
        TurnAgent(1);
	return true;
	}
      case CSKEY_F6:
        {
        MoveAgent(1);
	return true;
	}
      case CSKEY_F7:
        {
        TurnAgent(2);
	return true;
	}
      case CSKEY_F8:
        {
        MoveAgent(-1);
	return true;
	}
      case CSKEY_UP:
	player.StartMovement (csVector3 (0, 0, 1),user);
	return true;
      case CSKEY_DOWN:
	player.StartMovement (csVector3 (0, 0, -1),user);
	return true;
      case CSKEY_LEFT:
	player.StartRotation (csVector3 (0, -1, 0));
	return true;
      case CSKEY_RIGHT:
	player.StartRotation (csVector3 (0, 1, 0));
	return true;
      default:
        return handleKeyboardText(code);
    }
  }
  return false;
}
    
bool Game::handleKeyboardText(utf32_char code) {
  if (((code >= 'a') && (code <='z')) || ((code >='A') && (code <='Z'))) {
    userCommand.Append(code);
    return true;
  }
  else if ((code == ' ') || (code == '.') ||(code == '?')) {
    userCommand.Append(code);
    return true;
  }
  else switch (code) 
    { 
    case CSKEY_ENTER:
      sendAgentCommandToCANT();
      userCommand.Clear();
      return true;
    case '1':
      app->ReportError(userCommand);
      return true;
    case CSKEY_BACKSPACE:
      int length = userCommand.Length();
      userCommand.DeleteAt(length-1,1);
      return true;
    }
  return false;
}

csTicks startTime = 0;
csTicks occasionTime = 1000;  //once a second
csTicks lastTime = occasionTime*-1;
int lastAgentCommandTime = 0;
int backLastAgentCommandTime=0;

void Game::sendAgentCommandToCANT () {
  csRef<iVirtualClock> vc = csQueryRegistry<iVirtualClock>(app->
    GetObjectRegistry());

  if (!vc) {
     app->ReportError("Failed to locate Virtual Clock!");
     return;
  }

  //get the directory
  csRef<iVFS> vfs = csQueryRegistry<iVFS>(app->GetObjectRegistry());
        
  //remember to use the vfs mechanism via vfs.cfg
  //if  (!vfs->ChDir("/this/"))
  if  (!vfs->ChDir("/CABot3/"))
    {
    app->ReportError("Could not set directory!");
    return;
    }       

  //write the text to the file
  lastTime = vc->GetCurrentTicks ();
  scfString textOutput;
  char timeStr[20];
  textOutput.Append("start time ");
  sprintf(timeStr, "%d\n", startTime);
  textOutput.Append(timeStr);
  textOutput.Append("current time ");
  sprintf(timeStr, "%d\n", lastTime);
  textOutput.Append(timeStr);
  textOutput.Append(userCommand);
  if(!vfs->WriteFile("csToCABotText.txt", textOutput.GetData(), textOutput.Length()))
    {
    app->ReportError("Could not write text command!");
    return;
    }       
  if  (!vfs->Sync())
    {
    app->ReportError("Could not sync after write to CANT!");
    return;
    }       
}


//-----------------------------------------------------------------------------


void Game::PrintPictureOccasionally (csTicks time) {
  if ((lastTime+occasionTime)<time)
    {
      //printPicture(time);
    PrintPicture();
    }
}

//dump a camera picture to a jpg.
bool Game::PrintPicture() {

  csRef<iGraphics3D> g3d = csQueryRegistry<iGraphics3D>(app->GetObjectRegistry());
  if (!g3d) return app->ReportError("Failed to locate 3D renderer!");

  csRef<iGraphics2D> g2d = g3d->GetDriver2D ();

  csRef<iImage> img;
  csRef<iImageIO> imgio = csQueryRegistry<iImageIO>(app->GetObjectRegistry());
  csRef<iDataBuffer> data;
  csRef<iVFS> vfs = csQueryRegistry<iVFS>(app->GetObjectRegistry());
  //this was 14 with no crashing, but crashed when I got over
  //10 screens. It actually has 16 chars, so 18 sorts the problem
  //but will probably fail when we get above 1000.
  char str[18];
  static short int num = 0;       

  agentView->Draw();
  //grab the screenshot as a databuffer
  img = g2d->ScreenShot();       

  data = imgio->Save(img, "image/jpg");
  //remember to use the vfs mechanism via vfs.cfg
  //if  (!vfs->ChDir("/this/"))
  if  (!vfs->ChDir("/CABot3/"))
    {
    app->ReportError("Could not set directory!");
    return false;
    }       

  //it was quite a hassle to get the file working.  
  //I found the picture stuff on the net (search for screenshot in 
  //sourceforge's developers forum.
  //I found
  //if  (vfs->Exists("VFS.Mount.this"))
  //  csRef<iStringArray> mounts = vfs->GetMounts();
  //helped slog through it. CRH

  sprintf(str, "screenshot%d.jpg", num);
  //sprintf(str, "screenshot.png");
  
  //Write the data to a file...`
  //csRef<iFile> temp = vfs->Open("bob.txt", VFS_FILE_WRITE);
  if(!vfs->WriteFile(str, data->GetData(), data->GetSize()))
    {
    app->ReportError("Could not save screenshot!");
    return false;
    }       
  
  //app->ReportError ("Printing Picture");
  //write command file to CANT23Agent
  csRef<iVirtualClock> vc = csQueryRegistry<iVirtualClock>(app->
    GetObjectRegistry());
  lastTime = vc->GetCurrentTicks ();
  //lastTime = time;
  scfString textOutput;
  char timeStr[20];
  textOutput.Append("start time ");
  sprintf(timeStr, "%d\n", startTime);
  textOutput.Append(timeStr);
  textOutput.Append("current time ");
  sprintf(timeStr, "%d\n", lastTime);
  textOutput.Append(timeStr);
  sprintf(timeStr, "screenshot%d.jpg", num++);
  textOutput.Append(timeStr);
  if(!vfs->WriteFile("csToCABotPicture.txt", textOutput.GetData(), textOutput.Length()))
    {
    app->ReportError("Could not write command!");
    }       
  return true;
}

void Game::InterpretOneAgentCommand(char* command) {
  if (strcmp(command,"Left") == 0) {
    ExecuteAgentCommand(1);
  }
  else if (strcmp(command,"Right") == 0) {
    ExecuteAgentCommand(2);
  }
  else if (strcmp(command,"Forward") == 0) {
    ExecuteAgentCommand(3);
  }
  else if (strcmp(command,"Back") == 0) {
    ExecuteAgentCommand(4);
  }
  else if (strcmp(command,"Reset") == 0) {
    ExecuteAgentCommand(5);
  }
  else if (strcmp(command,"Reset2") == 0) {
    ExecuteAgentCommand(6);
  }
  else {
    char outStr[50];
    sprintf(outStr, "Bad Command From CABot3 %s\n", command);
    //app->ReportError(outStr);
  }
}

int Game::ProcessAgentCommand() {
  csRef<iVFS> vfs = csQueryRegistry<iVFS>(app->GetObjectRegistry());
  csRef<iDataBuffer> agentData;
  int notEOF;
        
  //remember to use the vfs mechanism via vfs.cfg
  if  (!vfs->ChDir("/CABot3/"))    {
    app->ReportError("Could not set directory!");
    return(0); }       

  agentData = vfs->ReadFile("cABotToCSCommand.txt", 1);
  //has this opened correctly?
  if (!agentData.IsValid())    {
    //ReportError("Could not read  to agent file; probably not closed!");
    return(0);    }       

  if  (!vfs->Sync())    {
    app->ReportError("Could not sync after agent read!");
    return(0);    }       

  //get the time and if time is less than or equal to the last
  //processed time return 0 to ignore
  char * text=new char(100);
  text = agentData->GetData();

  char c1,c2,c3,c4,c5,c6;
  int timeOfCommand;
  char* command = new char(10);
  notEOF = sscanf(text,"%c%c%c%c%c%c%d%s",&c1,&c2,&c3,&c4,&c5,&c6,&timeOfCommand,command);

  //delete []text;
  
  if (timeOfCommand <= lastAgentCommandTime)    {
    //delete []command;
    return(0);    }
 
  if ((notEOF < 7) || (timeOfCommand > 10000000))    {
    //delete []command;
    //app->ReportError("hmm");

    return(0);    }
 

  backLastAgentCommandTime=lastAgentCommandTime;
  lastAgentCommandTime=timeOfCommand;

  //skip the time 
  text = text+6;
  //find the next space to skip the time value e.g. 1234
  while ((text[0]!= ' ') && (text[0]!= '\n')) text++;
  //find the next space to skip the first command e.g. left
  while ((text[0]!= ' ') && (text[0]!= '\n')) text++;

  while(notEOF > 0) {
    InterpretOneAgentCommand(command);
    //crh app->ReportError(command);
    text++;
    //find the next space to skip the first command e.g. left
    while ((text[0]!= ' ') && (text[0]!= '\n')) text++;
    notEOF = sscanf(text,"%s",command);
  }

//char * print=new char(100);
//notEOF = sprintf(print,"test%d",strlen(text));
//notEOF = sprintf(print,"test%d",timeOfCommand);
//notEOF = sprintf(print,"test%d",27);
// if (notEOF > 0)
//app->ReportError(print);
// else 
//app->ReportError("noteof");

  return (0);
  }
